import { resolve } from 'path';
const isStringArray = (a) => Array.isArray(a) && !a.some(s => typeof s !== 'string');
export const getCoverageMap = async (config) => {
    const coverageMap = config.get('coverage-map');
    if (!coverageMap)
        return () => [];
    const map = (await import(resolve(config.globCwd, coverageMap)).catch(er => {
        throw new Error(`Coverage map ${coverageMap} is not a valid module. ${er.message}`);
    })).default;
    if (typeof map !== 'function') {
        throw new Error(`Coverage map ${coverageMap} did not default export a function`);
    }
    return (f) => {
        const mapped = map(f);
        if (mapped !== null &&
            typeof mapped !== 'string' &&
            !isStringArray(mapped)) {
            throw new Error(`Coverage map ${coverageMap} must return string, string[], or null`);
        }
        return mapped;
    };
};
//# sourceMappingURL=coverage-map.js.map