import { plugin as BeforePlugin } from '@tapjs/before';
import { plugin as SpawnPlugin } from '@tapjs/spawn';
import { plugin as StdinPlugin } from '@tapjs/stdin';
import { loaders } from '@tapjs/test';
import { glob } from 'glob';
import { relative, resolve } from 'node:path';
import { resolveImport } from 'resolve-import';
import { rimraf } from 'rimraf';
import { runAfter } from './after.js';
import { runBefore } from './before.js';
import { getCoverageMap } from './coverage-map.js';
import { executeTestSuite } from './execute-test-suite.js';
import { values } from './main-config.js';
import { outputDir } from './output-dir.js';
import { readSave } from './save-list.js';
import { testIsSerial } from './test-is-serial.js';
const piLoaderURL = await resolveImport('@tapjs/processinfo/esm', import.meta.url);
/* c8 ignore start */
if (!piLoaderURL) {
    throw new Error('could not get @tapjs/processinfo loader');
}
/* c8 ignore stop */
const piLoader = String(piLoaderURL);
const node = process.execPath;
export const run = async (args, config) => {
    // timeout in ms for test subprocesses, has a default
    // so it'll always be set to a number.
    /* c8 ignore start */
    const timeout = (config.get('timeout') ?? 30) * 1000;
    /* c8 ignore stop */
    // these all default to an empty array
    /* c8 ignore start */
    const testArgs = values['test-arg'] || [];
    const testEnv = values['test-env'] || [];
    const nodeArgs = values['node-arg'] || [];
    /* c8 ignore stop */
    process.env._TAPJS_PROCESSINFO_CWD_ = config.globCwd;
    process.env._TAPJS_PROCESSINFO_EXCLUDE_ = String(
    // don't consider snapshots and fixtures, or else we'll
    // always think that all tests are new after generating!
    /(^(node|tapmock):|[\\\/](tap-testdir-[^\\\/]+|tap-snapshots)[\\\/])/);
    const argv = [
        '--no-warnings=ExperimentalLoader',
        ...loaders.map(l => `--loader=${l}`),
        '--enable-source-maps',
        `--loader=${piLoader}`,
        ...nodeArgs,
    ];
    const env = { ...process.env };
    for (const e of testEnv) {
        if (!e.includes('=')) {
            delete env[e];
        }
        else {
            const split = e.split('=');
            const k = split[0];
            const v = split.slice(1).join('=');
            env[k] = v;
        }
    }
    const saveList = await readSave(config);
    const map = await getCoverageMap(config);
    return executeTestSuite(args, config, (t) => {
        // always set in dev, and the applyPlugin behavior is well tested
        /* c8 ignore start */
        const a = t.pluginLoaded(SpawnPlugin)
            ? t
            : t.applyPlugin(SpawnPlugin);
        const b = a.pluginLoaded(BeforePlugin)
            ? a
            : a.applyPlugin(BeforePlugin);
        const c = b.pluginLoaded(StdinPlugin)
            ? b
            : b.applyPlugin(StdinPlugin);
        /* c8 ignore stop */
        return c;
    }, async (t) => {
        runBefore(t, argv, config);
        runAfter(t, argv, config);
        // don't delete old coverage if only running subset of suites
        if (!config.get('changed') && !saveList.length) {
            await rimraf(resolve(config.globCwd, '.tap/processinfo'));
            await rimraf(resolve(config.globCwd, '.tap/coverage'));
        }
        outputDir(t, config);
    }, async (t, file, files, hasReporter) => {
        /* c8 ignore start */
        const stdio = hasReporter ? 'pipe' : 'inherit';
        /* c8 ignore end */
        if (file === '/dev/stdin') {
            if (files.length === 1)
                return t.stdinOnly();
            else
                return t.stdin();
        }
        const name = relative(config.globCwd, file);
        const mapped = map(name);
        let coveredFiles = mapped === null
            ? null
            : await glob(mapped, {
                cwd: config.globCwd,
                absolute: true,
            });
        const _TAPJS_PROCESSINFO_COVERAGE_ = coveredFiles === null ? '0' : '1';
        const _TAPJS_PROCESSINFO_COV_FILES_ = (coveredFiles || []).join('\n');
        const buffered = !testIsSerial(file) && t.jobs > 1;
        const args = [...argv, file, ...testArgs];
        return t.spawn(node, args, {
            at: null,
            stack: '',
            buffered,
            timeout,
            stdio,
            env: {
                ...env,
                _TAPJS_PROCESSINFO_COVERAGE_,
                _TAPJS_PROCESSINFO_COV_FILES_,
            },
            name,
            cwd: config.globCwd,
            externalID: name,
        });
    });
};
//# sourceMappingURL=run.js.map